﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    internal class WallMap : AbstractMap
    {
        /// <summary>
        /// Цвет участка закрытого
        /// </summary>
        private readonly Brush barrierColor = new SolidBrush(Color.Brown);
        /// <summary>
        /// Цвет участка открытого
        /// </summary>
        private readonly Brush roadColor = new SolidBrush(Color.LightPink);
        protected override void DrawBarrierPart(Graphics g, int i, int j)
        {
            g.FillPolygon(barrierColor, new PointF[]
            {
                new PointF(i       * _size_x, j * _size_y),
                new PointF((i + 1) * _size_x, j * _size_y),
                new PointF((i + 1) * _size_x - _size_x / 4, (j + 1) * _size_y - _size_y / 2),
                new PointF((i + 1) * _size_x, (j + 1) * _size_y),
                new PointF(i * _size_x, (j + 1) * _size_y),
                new PointF(i * _size_x + _size_x / 4, (j + 1) * _size_y - _size_y / 2),
            });
        }

        protected override void DrawRoadPart(Graphics g, int i, int j)
        {
            g.FillRectangle(roadColor, i * _size_x, j * _size_y, _size_x, _size_y);
        }

        protected override void GenerateMap()
        {
            _map = new int[120, 120];
            var minSize = Math.Min(_map.GetLength(0), _map.GetLength(1));
            _size_x = (float)_width / _map.GetLength(0);
            _size_y = (float)_height / _map.GetLength(1);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    _map[i, j] = _freeRoad;
                }
            }
            for (var i = 0; i < 10; i++)
            {
                var lengthWall = _random.Next(3, minSize / 2);
                var incX = _random.Next(0, 2);
                var incY = 1 - incX;
                var left = _random.Next(0, _map.GetLength(0) - lengthWall);
                var top =  _random.Next(0, _map.GetLength(1) - lengthWall);
                for (var j = 0; j < lengthWall; j++)
                {
                    _map[left + incX * j, top + incY * j] = _barrier;
                }
            }

        }
    }
}
